//
// knight-path - shortest knight path between two squares
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <cstring>

#include <knight-path/solver/brute_force.h>
#include <knight-path/solver/maze.h>
#include <knight-path/solver/taxi.h>


struct table_t
{
    const char *name;
    solver::pointer (*create)(void);
};

static const table_t table[] =
{
    { "brute-force", &solver_brute_force::create },
    { "fastest", &solver_maze::create },
    { "maze", &solver_maze::create },
    { "taxi", &solver_taxi::create },
};


#define ARRAY_SIZE(a) (sizeof(a) / sizeof(a[0]))
#define END_OF(a) ((a) + ARRAY_SIZE(a))


solver::pointer
solver::factory(const char *name)
{
    for (const table_t *tp = table; tp < END_OF(table); ++tp)
    {
        if (0 == strcmp(name, tp->name))
        {
            return tp->create();
        }
    }
    std::cerr << "solver algorithm \"" << name << "\" unknown" << std::endl;
    exit(EXIT_FAILURE);
}


void
solver::list_algorithms(void)
{
    for (const table_t *tp = table; tp < END_OF(table); ++tp)
        std::cout << tp->name << std::endl;
}
