#!/bin/sh
#
# knight-path - shortest knight path between two squares
# Copyright (C) 2011 Peter Miller
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
#
here=`pwd`
test $? -eq 0 || exit 2
work=${TMP_DIR-/tmp}/$$

tear_down()
{
        cd $here
        rm -rf $work
}

pass()
{
        tear_down
        echo PASSED
        exit 0
}

fail()
{
        tear_down
        echo 'FAILED test of the knight-path functionality'
        exit 1
}

no_result()
{
        tear_down
        echo 'NO RESULT for test of the knight-path functionality'
        exit 2
}

trap "no_result" 1 2 3 15

bin=$here/${1-.}/bin
mkdir $work
test $? -eq 0 || no_result
cd $work
test $? -eq 0 || no_result

cat > test.in << 'fubar'
a8 b7
fubar
test $? -eq 0 || no_result

cat > test.ok << 'fubar'
path: B6 A4 C5 B7
  +---+---+---+---+---+---+---+---+
8 | 0 |   |   |   |   |   |   |   |
  +---+---+---+---+---+---+---+---+
7 |   | 4 |   |   |   |   |   |   |
  +---+---+---+---+---+---+---+---+
6 |   | 1 |   |   |   |   |   |   |
  +---+---+---+---+---+---+---+---+
5 |   |   | 3 |   |   |   |   |   |
  +---+---+---+---+---+---+---+---+
4 | 2 |   |   |   |   |   |   |   |
  +---+---+---+---+---+---+---+---+
3 |   |   |   |   |   |   |   |   |
  +---+---+---+---+---+---+---+---+
2 |   |   |   |   |   |   |   |   |
  +---+---+---+---+---+---+---+---+
1 |   |   |   |   |   |   |   |   |
  +---+---+---+---+---+---+---+---+
    a   b   c   d   e   f   g   h
fubar
test $? -eq 0 || no_result

$bin/knight-path -a maze -b test.in > test.out
if test $? -ne 0
then
    cat test.out
    fail
fi

diff test.ok test.out
test $? -eq 0 || fail

#
# The things tested here, worked.
# No other guarantees are made.
#
pass
